@@ -2,16 +2,9 @@ Nonlin <- function(functionCall){
     badCall <- charmatch(c("model.frame.default", "model.matrix.default"),
                            sapply(sys.calls(),
                                   function(x) as.character(x[[1]])[1]))
-    if (!all(is.na(badCall))) {
-        culprit <- gsub(".default", "",
-                        as.character(sys.calls()
-                                     [[min(badCall[!is.na(badCall)])]][[1]]))
-        stop(paste(culprit,
-                   "has called Nonlin() from the gnm package.\n", culprit,
-                   "can only handle Nonlin terms",
-                   "as part of the formula of a gnm object."))
-    }
-
+    if (any(!is.na(badCall)))
+        stop(paste("Nonlin terms are only valid in gnm models."))
+    
     functionCall <- match.call()$functionCall
     envir <- environment(eval(functionCall[[1]]))
     varMethod <- paste(as.character(functionCall[[1]]), "Variables", sep = "")
